------------------------------------------------------------------------------
-- Button Cloner by Natch (natch@brainlag.se)
------------------------------------------------------------------------------
--- Configuration
-- Do you want to clear slots that were empty in the config you're loading?
local emptyButtons = false
------------------------------------------------------------------------------
--
-- Don't edit below unless you know what you're doing.
--
------------------------------------------------------------------------------
local ME           = ...
local ADDON        = "Natch's Button Cloner"
local MAX_BUTTONS  = 120
------------------------------------------------------------------------------
local ndebug      = false
------------------------------------------------------------------------------
local f, player, spec, lastStore

f = CreateFrame("Frame")
f:SetScript("OnEvent", function(_, event, ...) f[event](f, ...) end)
f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("VARIABLES_LOADED")

function f:ADDON_LOADED(msg)
  if msg == ME then
    self:UnregisterEvent("ADDON_LOADED")

    _G.SLASH_NATCH_BUTTONCLONER1 = "/nbc"
    _G.SLASH_NATCH_BUTTONCLONER2 = "/nbuttoncloner"
    _G.SLASH_NATCH_BUTTONCLONER3 = "/buttoncloner"
    _G.SlashCmdList["NATCH_BUTTONCLONER"] = function(...) self:SlashCMD(...); end

    player = UnitName("player").."@"..GetRealmName()

    self.ADDON_LOADED = nil
  end
end

function f:VARIABLES_LOADED()
  self:UnregisterEvent("VARIABLES_LOADED")

  if not BC_Data then
    BC_Data = {}
  end

  if not BC_Data[player] then
    BC_Data[player] = {}
  end

  self.db = BC_Data[player]

  self.VARIABLES_LOADED = nil
end

function f:StoreActionBars(spec)
  self:debug("StoreActionBars() spec: \"%s\"", spec)

  if not self.db[spec] then
    self.db[spec] = {}
    self.db[spec]["buttons"] = {}

    self:StoreButtons(spec)
  else
    self:OverwritePopup(spec)
  end
end

function f:StoreButtons(spec)
  self:debug("StoreButtons() spec: \"%s\"", spec)

  for buttonID = 1, MAX_BUTTONS do
    local btype, id, subType = GetActionInfo(buttonID)

    if btype and id and ndebug then
      self:debug("Button: %d | Type: %s | ID: %s | subType: %s", buttonID, btype or "N", id or "N", subType or "N")
    end

    self.db[spec]["buttons"][buttonID]      = {}
    self.db[spec]["buttons"][buttonID].type = btype or nil
    self.db[spec]["buttons"][buttonID].id   = id or nil
  end

  BC_Data[player] = self.db

  self:printf("Buttons stored to \"%s\".", spec)
end

function f:LoadActionBars(spec)
  self:debug("LoadActionBars() spec: \"%s\"", spec)

  if #self.db[spec]["buttons"] < 1 then
    self:printf("[ERROR]: Something is wrong with \"%s\" data.", spec)
    self:printf("  You may be able to salvage some of it by checking WTF\\Account\\<account>\\SavedVariables\\Natch_ButtonCloner.lua")
    return
  end

  self:CleanMouse(); -- Just in case we happen to have something in the cursor
                     -- when running this.

  local _iid = 1 -- internal id counter
  for buttonID, buttonData in pairs(self.db[spec]["buttons"]) do
    if buttonData.type == nil then
      if(emptyButtons == true) then
        PickupAction(_iid)
        ClearCursor()
      end
    elseif buttonData.type == "macro" then
      PickupMacro(buttonData.id)
    elseif buttonData.type == "spell" or buttonData.type == "companion" then
      PickupSpell(buttonData.id)
    elseif buttonData.type == "item" then
      PickupItem(buttonData.id)
    -- elseif buttonData.type == "flyout" then
      -- No support for these buttons yet.
    end

    if GetCursorInfo() then
      PlaceAction(buttonID)
    end

    -- In case we replaced something when placing the button we clear the
    -- cursor.
    self:CleanMouse()

    _iid = _iid + 1
  end

  self:printf("\"%s\" loaded.", spec)
end

function f:CleanMouse()
  if GetCursorInfo() then
    ClearCursor()
  end
end

-- Debug
function f:DumpDB(spec)
  self:debug("DumpDB() spec: \"%s\"", spec)

  for buttonID, buttonData in pairs(self.db[spec]["buttons"]) do
    self:printf("buttonID: %d | type: %s | id: %s", buttonID, buttonData.type or "None", tostring(buttonData.id) or "None")
  end
end

function f:VerifyLoad(spec)
  self:debug("VerifyLoad() spec: \"%s\"", spec)

  if not self.db[spec] then
    self:printf("[ERROR]: \"%s\" not found.", spec)
    return
  end

  StaticPopupDialogs["NATCH_BUTTONCLONER_VERIFYLOAD"] = {
    text           = "Are you sure you want to load \"%s\" buttons?",
    button1        = "Yes",
    button2        = "No",
    OnAccept       = function(_, data) f:LoadActionBars(data); end,
    OnCancel       = function() return; end,
    timeout        = 0,
    whileDead      = true,
    hideOnEscape   = false,
    preferredIndex = 3,
  }

  StaticPopup_Show("NATCH_BUTTONCLONER_VERIFYLOAD", spec, spec, spec)
end

function f:OverwritePopup(spec)
  self:debug("OverwritePopup() spec: \"%s\"", spec)

  StaticPopupDialogs["NATCH_BUTTONCLONER_OWPOP"] = {
    text           = "Do you want to overwrite \"%s\"?",
    button1        = "Yes",
    button2        = "No",
    OnAccept       = function(_, data) f:StoreButtons(data); end,
    OnCancel       = function(_, data) f:printf("[INFO]: Not overwriting \"%s\".", data); end,
    timeout        = 0,
    whileDead      = true,
    hideOnEscape   = false,
    preferredIndex = 3,
    showAlert      = true,
  }

  StaticPopup_Show("NATCH_BUTTONCLONER_OWPOP", spec, spec, spec)
end

function f:SlashCMD(str)
  self:debug("SlashCMD(): str: \"%s\"", str)

  local args = {}
  for m in string.gmatch(str, "[^ ]+") do
    table.insert(args, m)
  end

  local cmd = string.lower(args[1])

  if cmd == "store" or cmd == "save" then
    if #args == 2 then
      spec = args[2]
    else
      spec = self:GetSpec()
    end

    lastStore = spec
    self:StoreActionBars(spec)
  elseif cmd == "load" then
    if #args == 2 then
      spec = args[2]
    else
      if lastStore then
        spec = lastStore
      else
        self:printf("[ERROR]: Nothing stored this session or no profile specified.")
        return
      end
    end

    self:VerifyLoad(spec)
  elseif cmd == "dump" then
    self:DumpDB(spec)
  elseif cmd == "dumpspec" or cmd == "ds" then
    self:printf("spec: \"%s\"", spec)
    self:printf("GetSpec(): \"%s\"", f:GetSpec())
  else
    self:printf("Usage: /nbc store [<profile>]")
    self:printf("       /nbc load [<profile>]")
    self:printf("       /nbc help")
  end
end

function f:printf(s, ...)
  local str = string.format("%s: ", ADDON) .. s:format(...)
  SELECTED_CHAT_FRAME:AddMessage(str)
end

function f:debug(s, ...)
  if not ndebug then return; end
  self:printf("DEBUG: %s", s:format(...))
end

function f:GetSpec()
  local spec = select(2, GetSpecializationInfo(GetSpecialization()))
  return spec
end
